var menudata={children:[
{text:"wolfSSL Home",url:"https://www.wolfssl.com/",style:"clear:right;"},
{text:"Doc Home",url:"index.html"},
{text:"Related Pages",url:"pages.html"},
{text:"Modules",url:"modules.html"},
/*{text:"Data Structures",url:"annotated.html"},*/
{text:"Files",url:"files.html"}]}
